//
//	LINKFF.CPP
//	Link Folder to File
//
//	Given a folder path, creates a .XNK file which can be used to
//	launch an Exchange viewer on that folder, or attached to a
//	message as a folder link. This is the same type of file as is
//	generated by the File.Create Shortcut command in Exchange.
//
//	Command line arguments are as follows:
char szUsage[] = 
	"Usage: linkff folder-path file.xnk\r\n"
	"Examples:\r\n"
	"    linkff \"Public Folders\\Fishing Spots\" fishing.xnk\r\n"
	"    linkff Inbox default.xnk\r\n";
//
//	Exchange will create a new window to view the folder if you either
//	double-click on the .XNK file in the File Manager or Explorer, or
//	launch Exchange manually using the /t command line switch, as
//		exchng32 /t foo.xnk 
//
//	The following information on links to messages and folders is not part
//	of the MAPI specification. It applies only to the Microsoft Exchange
//	client version 4.0. Other MAPI clients, as well as future versions
//	of the Microsoft Exchange client, may not be compatible with links
//	built according to the information supplied here.
//

#include <windows.h>
#include <mapix.h>
#include <mapiutil.h>
#include <stdio.h>
#include <string.h>
#include "_links.h"

int
main(int argc, char *argv[])
{
	HRESULT			hr;
	BOOL			fMAPIInit = FALSE;
	LPMAPISESSION	pses = NULL;
	LPENTRYLIST		pel = NULL;
	LPSTREAM		pstream = NULL;
	LPSTR			szPath = NULL;

	if (argc != 3)
		goto usage;

	//	The usual startup nonsense
	if (hr = MAPIInitialize(0))
		goto ret;
	fMAPIInit = TRUE;
	if (hr = MAPILogonEx(0, NULL, NULL, MAPI_USE_DEFAULT | MAPI_NO_MAIL,
		&pses))
		goto ret;

	//	Get a list of entry IDs corresponding to the folder path
	//	from the command line. Use a copy of the path, because
	//	TraceFolderPath destroys the string.
	szPath = _strdup(argv[1]);
	if (hr = TraceFolderPath(pses, szPath, &pel, NULL))
	{
		printf("Failed to follow folder path %s\r\nMAPI error 0x%08lx\r\n",
			argv[1], GetScode(hr));
		goto ret;
	}

	//	Prepare to write the link data to the file named on the
	//	command line. 
	//	Alternatively, the stream passed to WriteFolderLink()
	//	could have been opened on the PR_ATTACH_DATA_BIN
	//	property of an attachment in a message; this would insert
	//	a folder link into the message.
	if (hr = OpenStreamOnFile(MAPIAllocateBuffer, MAPIFreeBuffer,
			STGM_READWRITE | STGM_CREATE, argv[2], "", &pstream))
	{
		printf("Failed to create link file %s\r\nMAPI error 0x%08lx\r\n",
			argv[2], GetScode(hr));
		goto ret;
	}

	//	Place the folder link information on the stream.
	if (hr = WriteFolderLink(pel, pstream))
	{
		printf("Failed to create link file %s\r\nMAPI error 0x%08lx\r\n",
			argv[2], GetScode(hr));
		goto ret;
	}

	//	Save the stream.
	if (hr = pstream->Commit(0))
	{
		printf("Failed to create link file %s\r\nMAPI error 0x%08lx\r\n",
			argv[2], GetScode(hr));
		goto ret;
	}

ret:
	//	Clean up after ourselves...
	UlRelease(pstream);
	MAPIFreeBuffer(pel);
	UlRelease(pses);
	if (fMAPIInit)
		MAPIUninitialize();
	if (szPath)
		free(szPath);

	//	Do the usual exit status thing: 
	//	0 => OK, nonzero => failure
	return ResultFromScode(hr) != S_OK;

usage:
	printf(szUsage);
	return 1;
}
