//
//	_LINKS.H
//	
//	Structures used in MS Exchange folder and message links
//
//	The following information on links to messages and folders is not part
//	of the MAPI specification. It applies only to the Microsoft Exchange
//	client version 4.0. Other MAPI clients, as well as future versions
//	of the Microsoft Exchange client, may not be compatible with links
//	built according to the information supplied here.
//


//
//	Folder link structures and constants
//

//	
//	The FVWIN structure describes how the Exchange viewer should appear.
//	
typedef struct
{
	ULONG	ulVersion;		// must be FVWIN_VERSION
	LONG	sw;				// ShowWindow state - SW_SHOWNORMAL etc.

	// Extent of window's rectangle, in pixels. xLeft and yTop give the
	// position of the upper left-hand corner; cx and cy give the width
	// and length respectively.
	// xSplit gives the position of the split bar between the folder and
	// message panes of the viewer. Even if you do not initially display
	// the folder pane (fTree == 0), choose a reasonable value for xSplit
	// because the user can make the folder pane visible through the UI.
	LONG	xLeft;
	LONG	yTop;
	LONG	cx;
	LONG	cy;
	LONG	xSplit;

	LONG	fTree;			// are we showing the folder pane (0 means no)?
	LONG	fToolbar;		// are we showing the toolbar (0 means no)?
	LONG	fStatusbar;		// are we showing the status bar (0 means no)?
} FVWIN;

#define FVWIN_VERSION	5

//
//	The FLINK structure describes the header part of a folder link file.
//	The most interesting part is the list of entry IDs which follows the header.
//
typedef struct 
{
	ULONG				ulVersion;		// FLINK_VERSION
	ULONG				ulObjectType;	// must be MAPI_FOLDER
	ULONG				cbIconData;		// must be zero
	FVWIN				fvwin;			// Window data

	//	Following this is the flattened folder path.
	//	The flattened form is as follows:
	//		length of the whole list (NOT including count)
	//		for each entry ID in the list:
	//			length of the node (INCLUDING count)
	//			entry ID type (MAPI_STORE or MAPI_FOLDER)
	//			actual size of the entry ID
	//			entry ID data
	//			pad bytes (garbage fill, 4-7 bytes as necessary to fill out to
	//			a multiple of 4)
	//		two zero bytes as a terminator
} FLINK;

#define FLINK_VERSION	5

//
//	Message link structures and constants
//

//
//	Utility data and functions
//

//	Property indexes for the prop value array returned from TraceMessage
enum { ivalSubject_TM = 0, ivalEntryID_TM };

//	Accommodate either forward or backward slashes in the path argument.
#define szPathSep "\\/"

//		FLDLINK.CPP
STDAPI	TraceFolderPath(LPMAPISESSION pses, LPSTR szPath,
		LPENTRYLIST FAR *ppel, LPMDB FAR *ppmdb);
STDAPI	WriteFolderLink(LPENTRYLIST pel, LPSTREAM pstream);
STDAPI	DupBin(LPSBinary pDst, LPSBinary pSrc, LPVOID pv);
STDAPI	TraceFolderNode(LPMAPITABLE ptbl, LPSTR FAR *szNode, LPSBinary pbin,
		LPVOID pv);
void	FlattenFolderPath(LPENTRYLIST pel, LPBYTE pb, int cb);
int		CbFromPel(LPENTRYLIST pel);

//		LINKUTL.CPP
STDAPI	OpenInbox(LPMAPISESSION pses, LPMDB FAR *ppmdb,
		LPMAPIFOLDER FAR *ppfld);
STDAPI	GetIconMetafile(HICON hicon, ULONG FAR *pcb, LPBYTE FAR *ppb);
STDAPI	GetIconStringMetafile(HICON hicon, LPSTR sz,
		ULONG FAR *pcb, LPBYTE FAR *ppb);
STDAPI	SetAttachmentIcon(HICON hicon, LPATTACH patch);
STDAPI	SetStgRendering(HICON hicon, LPSTR sz, LPSTORAGE pstg,
		LPOLESTR szStreamName);
STDAPI	PutStgStream(LPSTORAGE pstg, LPOLESTR szStreamName,
		ULONG cb, LPBYTE pb);

//		MSGLINK.CPP
STDAPI	TraceMessage(LPMAPIFOLDER pfldLink, LPSTR szSubject,
		LPSPropValue FAR *ppval);
STDAPI	WriteMessageLinkStg(LPSTORAGE pstg, LPSBinary pbinStore,
		LPSBinary pbinFolder, LPSBinary pbinMessage, LPSTR szSubject,
		HANDLE hicon);
STDAPI	WriteMessageLink(LPMESSAGE pmsg, LPSBinary pbinStore,
		LPSBinary pbinFolder, LPSBinary pbinMessage, LPSTR szSubject,
		HANDLE hicon, ULONG ulPos);
STDAPI	LinkMessageToMessage(LPMESSAGE pmsgLink, LPMESSAGE pmsgOuter, 
		HANDLE hicon, ULONG ulPos);

